module i2c_reg(
	clk,
	rstn,
    clk_10m,
	l1_lfps_det,
	lfps_det_dly,
	//l1_lfps_det_d3,
	// i2c control interface
	addr,
	write,
	datain,
	dataout,

	// IP interface
//	FUSEIN,
//	FUSEEN,
//	CLK_TEST_EN,

	A_POLLING,
	B_POLLING,

	A_DATAA_ENV_OUT,
	B_DATAB_ENV_OUT,

	A_TX_CTL,
	B_TX_CTL,

	A_HS_EN,
	B_HS_EN,

	A_LS_EN,
	B_LS_EN,

	A_RX_GAIN_OFF,
	B_RX_GAIN_OFF,

	A_TX_CMP,
	B_TX_CMP,

	A_RTERM_DETB,
	B_RTERM_DETB,

	U2_EN,
	A_RX_DET_CTL,
	A_RTERM_CMP_OFF,
	B_RX_DET_CTL,
	B_RTERM_CMP_OFF,
	LP_EN,
	DET_MOD,
	I2C_VTH_SWITCH,

	A_C1,
	A_C0,

	A_R1,
	A_R0,

	B_C1,
	B_C0,

	B_R1,
	B_R0,

	LDO_CTL,
	OSC_FAST,
	OSC_SLOW,
	VOD_CTL,

	RX_RTERM_CTL,
        TX_RTERM_CTL,
	RX_DET_CTL_SEL,

	LFPS_LVL,
	CFG_LOS_VTH,

	B_DE,
	A_DE,
	
        PU_CTL,
        LP_EN_CTL,
        DATA_EN_CTL,
	LS_EN_CTL,
	DE_LFPS_CTL,
        LS_DLY_MAX,
        LS_DLY_MIN,
        
	CM_CTLB_CTL,
        VCM_CTL,
	reg_en,
	data_env_en,
	d240ms_disable,
	nolfps_4ms_disable,
	rterm_detb_enter_u0_en,
	IBIAS_CTL_UP,
	IBIAS_CTL_DW

	);

input clk;
input rstn;
input clk_10m;
input l1_lfps_det;
//output l1_lfps_det_d3;
output [3:0] lfps_det_dly;

input [7:0] addr;
input write;
input [7:0] datain;

//input [55:0] FUSEIN;

//output reg FUSEEN;
//output reg CLK_TEST_EN;
output reg [7:0] dataout;

// Signal define
output        A_POLLING;
output        B_POLLING;

output        A_DATAA_ENV_OUT;
output        B_DATAB_ENV_OUT;

output        A_TX_CTL;
output        B_TX_CTL;

output        A_HS_EN;
output        B_HS_EN;

output        A_LS_EN;
output        B_LS_EN;

output        A_RX_GAIN_OFF;
output        B_RX_GAIN_OFF;

output        A_TX_CMP;
output        B_TX_CMP;

output        A_RTERM_DETB;
output        B_RTERM_DETB;

output        U2_EN;
output        A_RX_DET_CTL;
output        A_RTERM_CMP_OFF;
output        B_RX_DET_CTL;
output        B_RTERM_CMP_OFF;
output        LP_EN;
output        DET_MOD;
output        I2C_VTH_SWITCH;

output [3:0]    A_C1;
output [3:0]    A_C0;

output [3:0]    A_R1;
output [3:0]    A_R0;

output [3:0]    B_C1;
output [3:0]    B_C0;

output [3:0]    B_R1;
output [3:0]    B_R0;

output [1:0]    LDO_CTL;
output     OSC_FAST;
output     OSC_SLOW;
output [1:0]    VOD_CTL;

output [1:0]    TX_RTERM_CTL;
output [1:0]    RX_RTERM_CTL;
output [3:0]    RX_DET_CTL_SEL;

output [3:0]    LFPS_LVL;
output [2:0]    CFG_LOS_VTH;

output [3:0]    B_DE;
output [3:0]    A_DE;

output          PU_CTL;
output          LP_EN_CTL;

output [1:0]    DATA_EN_CTL;	
output [1:0]    LS_EN_CTL;
output          DE_LFPS_CTL;
output          LS_DLY_MAX;
output          LS_DLY_MIN;

output [2:0]    CM_CTLB_CTL;
output [4:0]    VCM_CTL;
output          reg_en;
output          data_env_en;
output          d240ms_disable;               
output          nolfps_4ms_disable;
output          rterm_detb_enter_u0_en;
output [2:0]    IBIAS_CTL_UP;
output [2:0]    IBIAS_CTL_DW;

// Reg define

reg [7:0] reg40;
reg [7:0] reg41;
reg [7:0] reg42;
reg [7:0] reg44;
reg [7:0] reg45;
reg [7:0] reg46;
reg [7:0] reg47;
reg [7:0] reg80;
reg [7:0] reg90;
reg [7:0] reg91;
reg [7:0] reg92;
reg [7:0] reg93;
reg [7:0] reg94;
reg [7:0] reg95;
//reg [2:0] l1_lfps_det_d;     

wire I2C_RST = 1'b0;

//assign l1_lfps_det_d3 = l1_lfps_det_d[2];

assign d240ms_disable = reg94[0];
assign nolfps_4ms_disable = reg91[0];   
assign rterm_detb_enter_u0_en  = ~reg41[0];

assign lfps_det_dly = reg95[4:1];


always @(negedge clk_10m or negedge rstn)
begin
	if(!rstn)
	    reg95[4:1] <= 4'h0;
	else
        reg95[4:1] <= {reg95[3:1], l1_lfps_det};
end

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg40 <= 8'h00;
	else if (I2C_RST)
		reg40 <= 8'h00;
	else if (write & (addr == 8'h40) )
		reg40 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg41 <= 8'h00;
	else if (I2C_RST)
		reg41 <= 8'h00;
	else if (write & (addr == 8'h41) )
		reg41 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg42 <= 8'h00;
	else if (I2C_RST)
		reg42 <= 8'h00;
	else if (write & (addr == 8'h42) )
		reg42 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg44 <= 8'h00;
	else if (I2C_RST)
		reg44 <= 8'h00;
	else if (write & (addr == 8'h44) )
		reg44 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg45 <= 8'h00;
	else if (I2C_RST)
		reg45 <= 8'h00;
	else if (write & (addr == 8'h45) )
		reg45 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg46 <= 8'h00;
	else if (write & (addr == 8'h46) )
		reg46 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg47 <= 8'h00;
	else if (I2C_RST)
		reg47 <= 8'h00;
	else if (write & (addr == 8'h47) )
		reg47 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg80 <= 8'h00;
	else if (I2C_RST)
		reg80 <= 8'h00;
	else if (write & (addr == 8'h80) )
		reg80 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg90 <= 8'h90;//8'h00;
	else if (I2C_RST)
		reg90 <= 8'h90;//8'h00;
	else if (write & (addr == 8'h90) )
		reg90 <= datain;

always @(negedge clk or negedge rstn )
        if (!rstn)
                reg91 <= 8'h04;//8'h26;
        else if (I2C_RST)
                reg91 <= 8'h04;//8'h26;
        else if (write & (addr == 8'h91) )
                reg91 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg92 <= 8'h00;
	else if (write & (addr == 8'h92) )
		reg92 <= datain;
	

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg93 <= 8'h4f;
	else if (I2C_RST)
		reg93 <= 8'h4f;
	else if (write & (addr == 8'h93) )
		reg93 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn)
		reg94 <= 8'hc0;
	else if (I2C_RST)
		reg94 <= 8'hc0;
	else if (write & (addr == 8'h94) )
		reg94 <= datain;

always @(negedge clk or negedge rstn )
	if (!rstn) begin
		reg95[0]   <= 0;
		reg95[7:5] <= 3'b000;
	end
	else if (I2C_RST) begin
		reg95[0]   <= 0;
		reg95[7:5] <= 3'b000;
	end
	else if (write & (addr == 8'h95) ) begin
		reg95[0]   <= datain[0];
        reg95[7:5] <= datain[7:5];
	end

// Datain & Dataout to IP
assign {A_POLLING, A_DATAA_ENV_OUT, A_TX_CTL, A_HS_EN,
	A_LS_EN, A_RX_GAIN_OFF, A_TX_CMP, A_RTERM_DETB} = reg40;
assign {B_POLLING, B_DATAB_ENV_OUT, B_TX_CTL, B_HS_EN,
        B_LS_EN, B_RX_GAIN_OFF, B_TX_CMP, B_RTERM_DETB} = reg41;
assign {U2_EN, LP_EN,A_RX_DET_CTL, A_RTERM_CMP_OFF, B_RX_DET_CTL, B_RTERM_CMP_OFF, DET_MOD,I2C_VTH_SWITCH} = reg42;

assign {RX_RTERM_CTL,TX_RTERM_CTL,RX_DET_CTL_SEL} = reg90;
assign {LFPS_LVL[3:0], CFG_LOS_VTH[2:0]} = reg91[7:1];
assign reg_en = reg80[0];//B_RTERM_DETB;//reg41[0];
assign data_env_en = ~reg80[1];//B_DATAB_ENV_OUT;
assign {B_DE[3:0], A_DE[3:0]} = reg92;
assign {IBIAS_CTL_UP[2:0], IBIAS_CTL_DW[2:0], PU_CTL, LP_EN_CTL } = reg93[7:0];

assign {A_C1, A_C0} = reg44;
assign {A_R1, A_R0} = reg45;
assign {B_C1, B_C0} = reg46;
assign {B_R1, B_R0} = reg47; 
assign {LDO_CTL[1:0], OSC_FAST, OSC_SLOW, VOD_CTL[1:0]} = reg80[7:2];
assign {DATA_EN_CTL,LS_EN_CTL,DE_LFPS_CTL,LS_DLY_MAX,LS_DLY_MIN} = reg94[7:1];
assign CM_CTLB_CTL = reg95[7:5];
assign VCM_CTL = {4'h0, reg95[0]};

always @(*)
	case (addr)
	// Dataout to I2C
		8'h40: dataout = reg40;
		8'h41: dataout = reg41;
		8'h42: dataout = reg42;
                8'h44: dataout = reg44;
                8'h45: dataout = reg45;
                8'h46: dataout = reg46;
                8'h47: dataout = reg47;
                8'h80: dataout = reg80;
		8'h90: dataout = reg90;
		8'h91: dataout = reg91;
		8'h92: dataout = reg92;
		8'h93: dataout = reg93;
		8'h94: dataout = reg94;
		8'h95: dataout =  {reg95[3:1], 4'h0, reg95[0]};
		default: dataout = 8'h00;
	endcase

endmodule


