//

module clk_rst(
	input CLK_H,
	input POR_RST,
	input FUSE_RST,
	output reg clk,
	output reg clk_div16,
	output reg clk_div6,
	output rstn
);

reg dl_rstn;
reg dl_rstn1;

reg clk_div4;
reg clk_div8;
//reg clk_div16;
reg clk_div32;
reg clk_div64;

wire rstn_fuse;
reg [13:0] rst_cntfuse;

always @(posedge CLK_H or negedge POR_RST)
        if (!POR_RST)
		rst_cntfuse <= 0;
	else if (rst_cntfuse < 10000) // 100ns * 10000 = 1ms
		rst_cntfuse <= rst_cntfuse + 1;

assign rstn_fuse =~( (rst_cntfuse >= 9998) & FUSE_RST);

// Synchronous reset to CLK_H
always @(posedge CLK_H or negedge POR_RST)
	if (!POR_RST)
		begin
			dl_rstn <= 1'b0;
			dl_rstn1 <= 1'b0;
		end
	else
		begin
			dl_rstn <= POR_RST;
			dl_rstn1 <= dl_rstn;
		end

assign rstn = dl_rstn1 & rstn_fuse;

// Clock divider
always @(posedge CLK_H or negedge POR_RST)
	if (!POR_RST)
		clk <= 1'b0;
	else
		clk <= ~clk;

always @(posedge clk or negedge POR_RST)
        if (!POR_RST)
		clk_div4 <= 1'b0;
	else
		clk_div4 <= ~clk_div4;

always @(posedge clk_div4 or negedge POR_RST)
        if (!POR_RST)
                clk_div8 <= 1'b0;
        else
                clk_div8 <= ~clk_div8;

always @(posedge clk_div8 or negedge POR_RST)
        if (!POR_RST)
                clk_div16 <= 1'b0;
        else
                clk_div16 <= ~clk_div16;

always @(posedge clk_div16 or negedge POR_RST)
        if (!POR_RST)
                clk_div32 <= 1'b0;
        else
                clk_div32 <= ~clk_div32;

always @(posedge clk_div32 or negedge POR_RST)
        if (!POR_RST)
                clk_div6 <= 1'b0;
        else
                clk_div6 <= ~clk_div6;

endmodule

