// USB control of one lane

module usb_lanectl_0 (
	input clk,
	input clk_div4,
	input rst_sel,
	input lfps_det_3cycle_1,
	output txpulse_cnt_rstn,
	input rterm_detb_n,
	input add_f_result,
	input sel_350_450,
	input  in_cnt_353ms_pulse,
	input  in_nolfps_4ms_pulse,
	output out_cnt_353ms_pulse,
	output pulse_350450_syn,
	input sel_4ms,
	input clk_div6,
	input rstn,
        input u2_rxdet_en_sel,
	input u2_rxdet_en_i,
	output reg             u2_rxdet_en,
	input powerdown,
	input data_env_en,
	input d240ms_disable,
        output nolfps_4ms_en,
        input [2:0]  cm_ctlb_ctl,
	input SLP_S0,
	output nolfps_4ms_r_n  ,
	input  o_nolfps_4ms_r_n,
	input [3:0]  rx_det_ctl_sel,

	input [1:0] i2c_rxdet_period,
	input i2c_u2_rxdet_interval,
	input [1:0]  i2c_u2_period,

	input u2_disable_rxdet,
	input lfps_debounce_en,

	input Polling,
	output U2_EN,

    input  usb_data_env_n,   
    output usb_data_env1_syn,
    output usb_data_env2_syn,

	input DATA_ENV_OUT,
	input data_env_sel,
	input HS_EN,
	input HS_EN_O,
	input LS_EN,
	input TX_CMP,

	input usb_en,
	input lfps_anotherchannel,
	input txtermdet_other,
	input [21:0] tout_cnt_other,
	input u2_nocmpdet_other,

	output lane1,
	output lfps_det_2,
	output reg txcmp_2cycle,
	output [21:0] tout_cnt,
	output u2_nocmpdet,

	output reg LP_EN,
	output reg CM_CTLB,
	output reg vddl_ctl,
	input      rterm_detb_ne0,
	output reg RX_DET_CTL,
	input      usb_rterm_cmp_off,
	output reg RTERM_CMP_OFF,
	output RX_GAIN_OFF,
	output reg RTERM_DETB,
	output reg RTERM_DETB_BUF,
	output reg HS_EN_BUF,
	output reg [12:0] txpulse_cnt,
    output reg  det_ctl0,
    output reg  det_ctl1,
    output reg  det_ctl2,
    output reg  det_ctl3,
    output reg  det_ctl4,
    output reg  det_ctl5,
    output reg  det_ctl6




);
//`define SIMANA

`ifdef SIMANA
parameter PULSE_START = 2;
parameter DET_PULSE_WIDTH_MUX = 505;
parameter DET_PULSE_WIDTH0 = 501;
parameter DET_PULSE_WIDTH1 = 401;
parameter DET_PULSE_WIDTH2 = 301;
parameter DET_PULSE_WIDTH3 = 201;
parameter DET_PULSE_WIDTH4 = 101;
parameter DET_PULSE_WIDTH5 = 51;
parameter DET_PULSE_WIDTH6 = 11;

parameter CM_DETB_WIDTH1 = 6;
parameter CM_DETB_WIDTH2 = 5;
parameter CM_DETB_WIDTH3 = 4;
parameter CM_DETB_WIDTH4 = 3;
parameter CM_DETB_WIDTH5 = 2;
parameter CM_DETB_WIDTH6 = 1;
parameter CM_DETB_WIDTH7 = 0;
`else
parameter PULSE_START = 33;
parameter DET_PULSE_WIDTH_MUX = 1010;
parameter DET_PULSE_WIDTH0 = 1001;
parameter DET_PULSE_WIDTH1 = 801;
parameter DET_PULSE_WIDTH2 = 601;
parameter DET_PULSE_WIDTH3 = 401;
parameter DET_PULSE_WIDTH4 = 201;
parameter DET_PULSE_WIDTH5 = 101;
parameter DET_PULSE_WIDTH6 = 21;

parameter CM_DETB_WIDTH1 = 102;
parameter CM_DETB_WIDTH2 = 89;
parameter CM_DETB_WIDTH3 = 77;
parameter CM_DETB_WIDTH4 = 64;
parameter CM_DETB_WIDTH5 = 51;
parameter CM_DETB_WIDTH6 = 38;
parameter CM_DETB_WIDTH7 = 25;


`endif



reg lfps_det_3cycle;


reg [15:0] txcmpinterval_cnt;

wire lfps_det = DATA_ENV_OUT; //LS_EN & DATA_ENV_OUT;

wire hs_det = data_env_en & DATA_ENV_OUT;  //HS_EN & DATA_ENV_OUT;

wire hs_det_1 = HS_EN_O;

reg pm_dl, pm_dl1, pm_dl2;

reg lfps_dl, lfps_dl1, lfps_dl2;

reg [1:0] lfps_det_cnt ;
reg lfps_det_dl1;
reg lfps_det_dl2;

wire lp_tout;
wire txdet_open;
wire rxterm_open;

reg u2_cmpreg;

reg [1:0] ctlbrst_cnt;

 // 250/ms

reg ctlb_dl1,ctlb_dl2; //ctlb_dl3;

reg txcmp_delay,txcmp_delay1,txcmp_delay2;
reg txcmp_det;
reg [1:0] txcmp_cnt;

reg RX_GAIN_OFF1;

wire lfps_wait;

wire [1:0] power_mode;

wire elec_idle;

wire lfps_u2debounce;

//add
reg  [9:0]      det_ctl_cnt;
wire            det_ctl_start;


reg             det_ctl_start_r;
reg             det_ctl;
reg [6:0]       cm_ctlb_width;
reg             cm_ctlb_ctl_en;

reg             rterm_detb_r;

reg             rx_det_ctl_tmp;
reg             rterm_cmp_off_tmp;

reg             clk_div2;
wire            u2_nocmpdet_tmp;

//reg             u2_en_r;

reg [2:0]       lfps_det_dly;
wire            lfps_det_0;

wire            u2_rxdet_en_0;

//wire            txpulse_cnt_rstn;

wire [3:0]      curr_state;

wire            txcmp_det_rstn;

//reg  [1:0]      l1_data_env_n_syn;
//reg  [1:0]      l2_data_env_n_syn;


assign usb_data_env1_syn = ~ det_ctl_cnt[8];
assign usb_data_env2_syn = ~ det_ctl_cnt[4];



//eco
//
//
assign nolfps_4ms_en = ~det_ctl_cnt[7];

always @(posedge pulse_350450_syn or negedge rterm_detb_n)
begin
	if(!rterm_detb_n) det_ctl_cnt[7] <= 1;
	else det_ctl_cnt[7] <= 0;
end




always @(posedge clk or negedge rstn)
begin
	if(!rstn) begin
		det_ctl_cnt[1] <= 1'b1;
		det_ctl_cnt[4] <= 1'b1;
		det_ctl_cnt[8] <= 1'b1;
		det_ctl_cnt[9] <= 1'b1;
	end
	else begin
		det_ctl_cnt[1] <= usb_data_env_n;
        det_ctl_cnt[9] <= usb_data_env_n;
		det_ctl_cnt[4] <= det_ctl_cnt[1];
		det_ctl_cnt[8] <= det_ctl_cnt[9];
	end
end


assign u2_nocmpdet = ~(~(u2_nocmpdet_tmp | add_f_result));


//assign u2_rxdet_en_0 = (u2_rxdet_en_sel == 1'b1)? u2_rxdet_en_i : u2_rxdet_en;

assign u2_rxdet_en_0 = u2_rxdet_en_i;

always @(posedge clk_div6 or negedge rstn)
begin
    if(!rstn)
    begin
        cm_ctlb_width  <= CM_DETB_WIDTH1;
        cm_ctlb_ctl_en <= 1'b0;
    end
    else
    begin
        case(cm_ctlb_ctl)
            3'b000: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH1;
                   cm_ctlb_ctl_en <= 1'b0;
	    end
	    3'b001: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH1;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b010: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH2;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b011: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH3;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b100: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH4;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b101: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH5;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b110: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH6;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
            3'b111: begin
                   cm_ctlb_width  <= CM_DETB_WIDTH7;
                   cm_ctlb_ctl_en <= 1'b1;
	    end
	    default:begin
                   cm_ctlb_width  <= CM_DETB_WIDTH1;
                   cm_ctlb_ctl_en <= 1'b0;
	    end 


	endcase
    end
end


//assign elec_idle = !LS_EN & !HS_EN & !lfps_anotherchannel; 
assign elec_idle = !data_env_sel & !lfps_anotherchannel;

assign RX_GAIN_OFF = RX_GAIN_OFF1 & (lfps_det_cnt < 2'b10) & (~lfps_det_dly[2]) & (~lfps_det_dly[1]) & (~lfps_det_dly[0]);// & !lfps_det_3cycle;  
assign lane1 = RX_GAIN_OFF & rxterm_open;

// in U2 mode, pull up LP_EN
always @(posedge clk or negedge rstn)
        if (!rstn)
		LP_EN <= 1;
	else if (powerdown)
		LP_EN <= 1;
	else if (!usb_en)
		LP_EN <= 1'b0;
	else if (txdet_open)
		LP_EN <= 1;
	else
		LP_EN <= (RTERM_DETB || lp_tout) ;

assign U2_EN = (power_mode == 2'b11);// & (lfps_det_cnt == 2'b00);

// vddl_ctl drive by register
always @(posedge clk or negedge rstn)
	if (!rstn)
		vddl_ctl <= 1;
	else if (powerdown)
                vddl_ctl <= 1;
	else if (!usb_en)
                vddl_ctl <= 0;
	else
		vddl_ctl <= CM_CTLB;

`ifdef SIMANA
wire u2_rxdet_cntmatch = i2c_u2_rxdet_interval ? txpulse_cnt == 9 : txpulse_cnt == 8;

wire rxdet_cntmatch = i2c_rxdet_period == 2'b11 ? txpulse_cnt == 9 :
                        ( i2c_rxdet_period == 2'b10 ? txpulse_cnt == 9 :
                                txpulse_cnt == 8 );

wire rxdet_morematch = i2c_rxdet_period == 2'b11 ? txpulse_cnt == 10 :
                        ( i2c_rxdet_period == 2'b10 ? txpulse_cnt == 10 :
                                txpulse_cnt == 9 );

`else
// 6.4us  
wire u2_rxdet_cntmatch = i2c_u2_rxdet_interval ? txpulse_cnt == 9375 : txpulse_cnt == 7813;

wire rxdet_cntmatch = i2c_rxdet_period == 2'b11 ? txpulse_cnt == 1875 : 
			( i2c_rxdet_period == 2'b10 ? txpulse_cnt == 1250 :
				txpulse_cnt == 781 );

wire rxdet_morematch = i2c_rxdet_period == 2'b11 ? txpulse_cnt == 1925 :
			( i2c_rxdet_period == 2'b10 ? txpulse_cnt == 1400 :
				txpulse_cnt == 925 );

`endif

// Pull down RX terminal
always @(posedge clk or negedge rstn)
	if	(!rstn) begin
		RX_GAIN_OFF1	<=	1'b1;
		HS_EN_BUF	<=	1'b0;
		RTERM_DETB	<=	1'b1;
		RTERM_DETB_BUF	<=	1'b1;
	end
        else if (powerdown) begin
                RX_GAIN_OFF1     <=      1'b1;
                HS_EN_BUF       <=      1'b0;
                RTERM_DETB      <=      1'b1;
                RTERM_DETB_BUF  <=      1'b1;
        end
	else if	(!usb_en) begin
		RX_GAIN_OFF1	<=	1'b0;
		HS_EN_BUF	<=	1'b0;
		RTERM_DETB	<=	1'b0;
		RTERM_DETB_BUF	<=	1'b0;
	end
	else begin
		RX_GAIN_OFF1	<=	power_mode	!=	2'b01;	//	close	in	U0
		HS_EN_BUF	<=	HS_EN;	//	open	in	U0
		RTERM_DETB	<=	~	rxterm_open;
		RTERM_DETB_BUF	<=	~	rxterm_open;
	end

// Count LFPS cycles, if more than 3 cycles, send LFPS detect to State Machine


always @(posedge clk or negedge rstn)
        if      (!rstn) begin
		lfps_det_dly <= 3'h0;
	end
	else begin
		lfps_det_dly <= {lfps_det_dly[1:0],lfps_det_0}; 
        end


assign lfps_det_2 = lfps_det_dly[2];


assign lfps_det_0 = (lfps_det_cnt == 2)?  1'b1 : 1'b0;

assign rstn1 = rstn & ~lfps_det_3cycle;
always @(posedge Polling or negedge rstn1)
	if	(!rstn1)
		lfps_det_cnt	<=	2'b00;
	else if (powerdown | (lfps_det_3cycle))
                lfps_det_cnt    <=      2'b00;
	else if	(lfps_det_cnt	<	2'b10)
		lfps_det_cnt	<=	lfps_det_cnt	+	2'b01;

always @(posedge Polling or negedge rstn)
        if (!rstn) begin
		pm_dl <= 1'b0;
		pm_dl1 <= 1'b0;
	end
	else begin
		pm_dl <= (power_mode == 2'b01); 
		pm_dl1 <= pm_dl;
	end

always @(posedge clk or negedge rstn)
        if      (!rstn) begin
		lfps_det_dl1 <= 0;
		lfps_det_dl2 <= 0;
		lfps_det_3cycle <= 0;
	end
	else begin
                lfps_det_dl1 <=  lfps_det_cnt != 2'b00;
		lfps_det_dl2 <= lfps_det_dl1;
                lfps_det_3cycle <= lfps_det_dl2;
        end



//assign txpulse_cnt_rstn = (rstn & (~(curr_state == 4'b1000))) & (~((~U2_EN) & RX_DET_CTL));
assign txpulse_cnt_rstn = rstn & (~((~U2_EN) & RX_DET_CTL));

// Generate TX terminal detecting pulse
always @(posedge clk_div6 or negedge txpulse_cnt_rstn)
	if	(!txpulse_cnt_rstn)
		txpulse_cnt <=	13'h0000;
        else if (powerdown)
                txpulse_cnt <=  13'h0000;
	else if	( txdet_open != 1'b1 && power_mode !=	2'b11 )
		txpulse_cnt	<= 13'h0000;
	//else if((~U2_EN) & u2_en_r)
		//txpulse_cnt <= 13'h0000;
	else if( (txdet_open && rxdet_cntmatch ) //pulse 5ms=8us*625 for TX Detect
		|| u2_rxdet_cntmatch ) //period 50ms/60ms=for CM_CTLB
		txpulse_cnt <= 13'h0000;
	else
		txpulse_cnt <= txpulse_cnt +13'h0001;

always @(posedge clk_div6 or negedge rstn)
        if      (!rstn) begin
		ctlb_dl1 <= 0;
		ctlb_dl2 <= 0;
		//ctlb_dl3 <= 0;
	end 
	else begin
                ctlb_dl1 <= power_mode == 2'b01;
                ctlb_dl2 <= ctlb_dl1;
		//ctlb_dl3 <= ctlb_dl2;
        end

//assign RX_DET_CTL = (rx_det_ctl_tmp&(~U2_EN)) || (rx_det_ctl_tmp&u2_rxdet_en_0);

//assign RTERM_CMP_OFF = (rterm_cmp_off_tmp | U2_EN) & (rterm_cmp_off_tmp | (U2_EN & (~u2_rxdet_en_0)));


always @(posedge clk or negedge rstn)
    if(!rstn)    RTERM_CMP_OFF <= 1'b1;
	else         RTERM_CMP_OFF <= usb_rterm_cmp_off;

always @(posedge clk or negedge rstn)
	if (!rstn) begin
		rx_det_ctl_tmp <= 1'b0;
		rterm_cmp_off_tmp <= 1'b1;
		CM_CTLB <= 1'b1;
	end
        else if (powerdown) begin
                rx_det_ctl_tmp <= 1'b0;
                rterm_cmp_off_tmp <= 1'b1;
                CM_CTLB <= 1'b1;
        end
	else if (!usb_en) begin
		rx_det_ctl_tmp <= 1'b0;
		rterm_cmp_off_tmp <= 1'b1;
		CM_CTLB <=  1'b1;//ctlbrst_cnt >= 2'b10;
	end
	else begin
`ifdef SIMANA
               	rx_det_ctl_tmp <= (txdet_open || (!u2_disable_rxdet && power_mode == 2'b11) ) 
			& det_ctl;//((det_ctl&(~U2_EN)) || (det_ctl&U2_EN&u2_rxdet_en));//&& (txpulse_cnt < 5 ) & (txpulse_cnt >= 2) // pulse 100us = 6.4us * 16 in RX detect mode
	        	 
		rterm_cmp_off_tmp <= ~ ( ( txdet_open || (!u2_disable_rxdet && power_mode == 2'b11) )  & (txpulse_cnt < 4) & (txpulse_cnt >= 1) ); //

		CM_CTLB <= ~( power_mode == 2'b01 && ctlb_dl1 == 0 && ctlb_dl2 == 0 ) && // delay in U0
			( ( !txdet_open && power_mode == 2'b00 && !rxterm_open ) || // powerdown 
			(txdet_open ) || // detect state
			power_mode == 2'b01 ||  // in U0 State
	               (((~cm_ctlb_ctl_en) &(txpulse_cnt < 7 ) & (txpulse_cnt >= 6 )) // Send pulse in U2
		        || (cm_ctlb_ctl_en &(txpulse_cnt[6:0] >  cm_ctlb_width)))
		        );
`else
		
		rx_det_ctl_tmp <= (txdet_open || (!u2_disable_rxdet && power_mode == 2'b11) ) 
			& det_ctl;//((det_ctl&(~U2_EN)) || (det_ctl&U2_EN&u2_rxdet_en));//&& (txpulse_cnt < 50 ) & (txpulse_cnt >= 33) // pulse 100us = 6.4us * 16 in RX detect mode
	        	 
		rterm_cmp_off_tmp <= ~ ( ( txdet_open || (!u2_disable_rxdet && power_mode == 2'b11) )  & (txpulse_cnt < 49 ) & (txpulse_cnt >= 2) ); // pulse 300us = 6.4us * 47

		CM_CTLB <= ~( power_mode == 2'b01 && ctlb_dl1 == 0 && ctlb_dl2 == 0 ) && // delay in U0
			( ( !txdet_open && power_mode == 2'b00 && !rxterm_open ) || // powerdown 
			(txdet_open ) || // detect state
			power_mode == 2'b01 ||  // in U0 State
	               (((~cm_ctlb_ctl_en) &(txpulse_cnt < 3016 ) & (txpulse_cnt >= 3000) ) // Send pulse in U2, 100us = 6.4us * 16
		        || (cm_ctlb_ctl_en &(txpulse_cnt[6:0] >  cm_ctlb_width)))
		        );
`endif
 	end



/////eco
/*
`ifdef SIMANA

always @(posedge txpulse_cnt[1] or negedge U2_EN)
begin
	if(!U2_EN)   u2_rxdet_en <= 0;
	else         u2_rxdet_en <= 1;
end
`else
always @(posedge txpulse_cnt[10] or negedge U2_EN)
begin
	if(!U2_EN)   u2_rxdet_en <= 0;
	else         u2_rxdet_en <= 1;
end

`endif
*/
/////eco


//rx_det_ctl_sel
//
assign det_ctl_start = (txpulse_cnt == PULSE_START);

always @(posedge clk or negedge rstn)
begin
	if(!rstn)
		det_ctl_start_r <= 0;
	else
		det_ctl_start_r <= det_ctl_start;
end




always @(posedge clk or negedge rstn)
begin
	if(!rstn) begin
		//det_ctl_cnt[7] <= 1; 
	        det_ctl_cnt[5] <= 1;	
		det_ctl_cnt[3:2] <= 2'b10;
		det_ctl_cnt[0]   <= 1'b1;	
	end
	else if((det_ctl_start_r == 1'b0)&&(det_ctl_start == 1'b1)) begin
                //det_ctl_cnt[7] <= 0; 
	        det_ctl_cnt[5] <= 0;  
		det_ctl_cnt[3:2] <= 2'b00;
		det_ctl_cnt[0]   <= 1'b0;	
	end
	else if(det_ctl_cnt < DET_PULSE_WIDTH_MUX)
		{det_ctl_cnt[5],det_ctl_cnt[3:2],det_ctl_cnt[0]} <= {det_ctl_cnt[5],det_ctl_cnt[3:2],det_ctl_cnt[0]} + 1;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH6))
		det_ctl6 <= 1'b1;
	else
		det_ctl6 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH5))
		det_ctl5 <= 1'b1;
	else
		det_ctl5 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH4))
		det_ctl4 <= 1'b1;
	else
		det_ctl4 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH3))
		det_ctl3 <= 1'b1;
	else
		det_ctl3 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH2))
		det_ctl2 <= 1'b1;
	else
		det_ctl2 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH1))
		det_ctl1 <= 1'b1;
	else
		det_ctl1 <= 1'b0;
end

always @(posedge clk)
begin
	if((det_ctl_cnt > 0) && (det_ctl_cnt < DET_PULSE_WIDTH0))
		det_ctl0 <= 1'b1;
	else
		det_ctl0 <= 1'b0;
end



always @(*)
begin
	case(rx_det_ctl_sel)
		4'b0000: det_ctl = det_ctl0;
		4'b0001: det_ctl = det_ctl1;
		4'b0010: det_ctl = det_ctl2;
		4'b0011: det_ctl = det_ctl3;
		4'b0100: det_ctl = det_ctl4;
		4'b0101: det_ctl = det_ctl5;
		4'b0110: det_ctl = det_ctl6;
		default: det_ctl = det_ctl0;
	endcase
end






// Detect hot plug in U2
always @(posedge clk or negedge rstn)
        if (!rstn)
		u2_cmpreg <= 0;
        else if (powerdown) 
                u2_cmpreg <= 0;
	else if ( (power_mode != 2'b11) ) 
		u2_cmpreg <= 0;
`ifdef SIMANA
        else if (txpulse_cnt > 6)
`else
	else if (txpulse_cnt > 1000)
`endif
		u2_cmpreg <= 0;
	else if ( (power_mode == 2'b11) && (!TX_CMP))
		u2_cmpreg <= 1;

`ifdef SIMANA
assign u2_nocmpdet_tmp = (power_mode == 2'b11) && (txpulse_cnt == 6) && (u2_cmpreg == 0) && u2_rxdet_en_0;

`else
assign u2_nocmpdet_tmp = (power_mode == 2'b11) && (txpulse_cnt == 200) && (u2_cmpreg == 0) && u2_rxdet_en_0;
`endif

// Sync TM_CMP to clk
always @(posedge clk or negedge rstn)
	if (!rstn)
		{txcmp_delay2, txcmp_delay1, txcmp_delay} <= 3'b111;
        else if (powerdown)
                {txcmp_delay2, txcmp_delay1, txcmp_delay} <= 3'b111;
	else if (!txdet_open)
		{txcmp_delay2, txcmp_delay1, txcmp_delay} <= 3'b111;
	else
		{txcmp_delay2, txcmp_delay1, txcmp_delay} <= {txcmp_delay1, txcmp_delay, TX_CMP};


//eco
/*
always @(posedge clk or negedge rstn)
begin
    if(!rstn)   rterm_detb_r <= 1;
    else        rterm_detb_r <= RTERM_DETB;

end
*/

/*
always @(posedge clk_div6 or negedge rstn)
begin
	if(!rstn)      u2_en_r <= 1'b0;
	else           u2_en_r <= U2_EN;
end
*/

always @(posedge clk or negedge rstn)
begin
	if(!rstn)      RX_DET_CTL <= 1'b0;
	else           RX_DET_CTL <= U2_EN;
end


//eco end

wire nolfps_4ms_pulse_r;
wire nolfps_4ms_rstn;


assign nolfps_4ms_r_n = det_ctl_cnt[6];


always @(posedge nolfps_4ms_pulse_r or negedge nolfps_4ms_rstn)
begin
    if(!nolfps_4ms_rstn)
        det_ctl_cnt[6] <= 1'b1;//sel_4ms;
    else
        det_ctl_cnt[6] <= 0;
end




assign txcmp_det_rstn = rstn & (~rterm_detb_ne0);

// TX_CMP active low for 2 times 
always @(posedge clk or negedge txcmp_det_rstn)
        if (!txcmp_det_rstn)
		txcmp_det <= 1'b0;
`ifdef SIMANA
        else if (powerdown || !usb_en || txpulse_cnt >= 7 ) //|| (rterm_detb_r & (~RTERM_DETB)))
`else
	else if (powerdown || !usb_en || txpulse_cnt >= 500 ) //|| (rterm_detb_r & (~RTERM_DETB)))
`endif
		txcmp_det <= 1'b0;
	else if ({txcmp_delay2, txcmp_delay1} == 2'b01 )
		txcmp_det <= 1'b1;

always @(posedge clk_div6 or negedge rstn)
	if (!rstn)
		txcmp_cnt <= 2'b00;
        else if (powerdown | !usb_en)
                txcmp_cnt <= 2'b00;
	else if (!txdet_open)
		txcmp_cnt <= 2'b00;
`ifdef SIMANA
        else if ( txpulse_cnt == 6 && txcmp_det == 1'b0 )
                txcmp_cnt <= 2'b00;
        else if ( txpulse_cnt == 6 && txcmp_det == 1'b1 && txcmp_cnt < 2'b10 )
                txcmp_cnt <= rxdet_morematch ? 2'b00 : (txcmp_cnt + 2'b01);
`else
	else if ( txpulse_cnt == 400 && txcmp_det == 1'b0 )
		txcmp_cnt <= 2'b00;
	else if ( txpulse_cnt == 400 && txcmp_det == 1'b1 && txcmp_cnt < 2'b10 )
		txcmp_cnt <= rxdet_morematch ? 2'b00 : (txcmp_cnt + 2'b01);
`endif

always @(posedge clk or negedge rstn)
	if (!rstn)
		txcmp_2cycle <= 1'b0;
        else if (powerdown)
                txcmp_2cycle <= 1'b0;
	else if (!usb_en || power_mode[1] || ( !txdet_open && power_mode == 2'b00 && !rxterm_open ) )
		txcmp_2cycle <= 1'b0;
	else if (txcmp_cnt == 2'b10 )
		txcmp_2cycle <= 1'b1;
	else if (txcmp_cnt == 2'b00 ) //diff
		txcmp_2cycle <= 1'b0;


// TX CMP counter
always @(posedge clk or negedge rstn)
	if (!rstn)
		txcmpinterval_cnt <= 16'h0000;
        else if (powerdown)
                txcmpinterval_cnt <= 16'h0000;
	else if (!txdet_open | TX_CMP)
		txcmpinterval_cnt <= 16'h0000;
	else 
		txcmpinterval_cnt <= txcmpinterval_cnt + 16'h0001;

always @(negedge clk_div6 or negedge rstn)
	if (!rstn)
		ctlbrst_cnt <= 2'b00;
        else if (powerdown)
                ctlbrst_cnt <= 2'b00;
	else if (ctlbrst_cnt <= 2'b11)
		ctlbrst_cnt <= ctlbrst_cnt + 2'b01;

power_st_0 u_power_st(
	.clk(clk),
	.clk_div4(clk_div4),
	.rst_sel(rst_sel),
	.lfps_det_3cycle_1(lfps_det_3cycle_1),
	.u2_en(U2_EN),
	.sel_350_450(sel_350_450),
	.sel_4ms(sel_4ms),
        .in_cnt_353ms_pulse (in_cnt_353ms_pulse ),
	.in_nolfps_4ms_pulse (in_nolfps_4ms_pulse),
        .nolfps_4ms_pulse_r  (nolfps_4ms_pulse_r   ) ,
        .nolfps_4ms_rstn     (nolfps_4ms_rstn      ) ,
        .out_cnt_353ms_pulse(out_cnt_353ms_pulse),
	.pulse_350450_syn(pulse_350450_syn),
	.rstn(rstn),
	.en(usb_en),
	.d240ms_disable(d240ms_disable),
	.nolfps_4ms_en (nolfps_4ms_en ),
        .data_env_out_sync(DATA_ENV_OUT),
	.data_env_out_sync_l1(usb_data_env1_syn),
        .SLP_S0(SLP_S0),
        
         .nolfps_4ms_r_n   (   ),  
         .o_nolfps_4ms_r_n (nolfps_4ms_r_n ),

	.i2c_u2_period(i2c_u2_period),

	// input signals from analog
	.lfps_det(lfps_det_2),//(lfps_det_3cycle ),
	.lfps_anotherchannel(lfps_anotherchannel),
	.lfps_u2debounce(lfps_u2debounce),

	.RTERM_DETB(RTERM_DETB),

	.txterm_det(txcmp_2cycle),
	.txterm_det_other(txtermdet_other),
	.hs_det(hs_det),
	.hs_det_1(hs_det_1),
	.elec_idle(elec_idle),
	.u2_nocmpdet(u2_nocmpdet),
	.u2_nocmpdet_other(u2_nocmpdet_other),
	.tout_cnt_other(tout_cnt_other),

	// output to analog
	.hs_open(),
	.lp_tout(lp_tout),
	.lfps_wait(lfps_wait),
	//.cma_note(cma_note),
	.power_mode(power_mode),
	.tout_cnt(tout_cnt),
	.txdet_open(txdet_open),
	.curr_state(curr_state),
	.rxterm_open(rxterm_open)
);

lfps_debounce u_lfps_debounce (
        .clk_div6(clk_div6),
        .rstn(rstn),
        .lfps_debounce_en(lfps_debounce_en), 
        .lfps_in(lfps_anotherchannel),
        .lfps_out(lfps_u2debounce)
        );

endmodule 

