// USB dual channel

module usb_rxtx (
        input clk,
        input clk_div16,
        input clk_div6,
        input rstn,
        input usb_rterm_cmp_off,
        input add_a_f_result,
        input add_b_f_result,
        input sel_350_450,
        input powerdown,

        input SLP_S0,
        input [2:0] cm_ctlb_ctl,
        input [3:0] rx_det_ctl_sel,
        input       det_mod,
        input       data_env_en,
        input       d240ms_disable,
        //input       nolfps_4ms_en,
        input       rterm_detb_enter_u0_en,
        input [1:0] i2c_rxdet_period,
        input i2c_u2_rxdet_interval,
        input [1:0] i2c_u2_period,
        input u2_disable_rxdet,
        input lfps_debounce_en,

        input Polling_1,
        input DATA_ENV_OUT_1,
        input HS_EN_1,
        input LS_EN_1,
        input TX_CMP_1,

        input Polling_2,
        input DATA_ENV_OUT_2,
        input HS_EN_2,
        input LS_EN_2,
        input TX_CMP_2,

        input usb_en,
        output l1_lfps_det,
        input [3:0] lfps_det_dly,
        output lfps_det_3cycle_2,
        output cm_active,

        output lane1,
        output lane2,
        output lane12,
        output LP_EN,
        output LP_EN_1,
        output LP_EN_2,
        output U2_EN,
        output U2_EN_1,
        output U2_EN_2,
        
        output RX_GAIN_OFF,

        output CM_CTLB_1,
        output vddl_ctl_1,
        output RX_DET_CTL_1,
        output RTERM_CMP_OFF_1,
        output RX_GAIN_OFF_1,
        output RTERM_DETB_1,
        output RTERM_DETB_BUF_1,
        output HS_EN_BUF_1,

        output CM_CTLB_2,
        output vddl_ctl_2,
        output RX_DET_CTL_2,
        output RTERM_CMP_OFF_2,
        output RX_GAIN_OFF_2,
        output RTERM_DETB_2,
        output RTERM_DETB_BUF_2,
        output HS_EN_BUF_2,
        output [12:0] txpulse_cnt,
        output pulse_350450_syn,
        input  u2_rxdet_en,
        output u2_rxdet_en_1,
        output u2_rxdet_en_2,
        output  tx_cmp_s,
        output  det_ctl0,
        output  det_ctl1,
        output  det_ctl2,
        output  det_ctl3,
        output  det_ctl4,
        output  det_ctl5,
        output  det_ctl6

);

//wire rterm_detb_s;
//wire  tx_cmp_s;
wire txpulse_cnt_rstn;
wire nolfps_4ms_en;

wire txcmp_2cycle_1;
wire txcmp_2cycle_2;

wire u2_nocmpdet_1, u2_nocmpdet_2;
wire [21:0] tout_cnt_1,tout_cnt_2;
//wire cma_note1,cma_note2;

reg  [1:0] data_env_out_sync;
wire  rterm_detb_lfps1;
wire  rterm_detb_lfps2;
reg   rterm_detb_d;
wire  rterm_detb_ne;
wire  rterm_detb_ne0;

wire  l1_4ms_s;
wire  l2_4ms_s;

wire  clk_div4;

wire  l1_353ms_pulse;
wire  l2_353ms_pulse;

reg   [3:0] div_cnt;

wire  rterm_detb_n;

wire usb_data_env_n;

wire usb_data_env1_syn;
wire usb_data_env2_syn;

wire l1_nolfps_4ms_pulse;

assign lfps_det_3cycle_1 = lfps_det_dly[2];

//assign RTERM_DETB_2 = (det_mod == 1'b1)? RTERM_DETB_1 : rterm_detb_s;

assign lane12 = lane1 & lane2;  // Two channel both in U2

assign LP_EN = LP_EN_1;// & LP_EN_2; // if both in U2, pull up LP_EN
assign U2_EN = U2_EN_1 & U2_EN_2;

assign RX_GAIN_OFF = RX_GAIN_OFF_1 & RX_GAIN_OFF_2;


assign rterm_detb_n = ~(RTERM_DETB_1 | RTERM_DETB_2);

assign tx_cmp_s = (det_mod)? TX_CMP_1 : TX_CMP_2;


assign usb_data_env_n = ~(DATA_ENV_OUT_1 | DATA_ENV_OUT_2);


always @(posedge clk or negedge rstn)
begin
        if(!rstn)   div_cnt <= 0;
        else        div_cnt <= div_cnt + 1;
end


assign clk_div4 = div_cnt[3];
 


always @(posedge clk)
begin
    rterm_detb_d <= RTERM_DETB_1 | RTERM_DETB_2;
end

assign rterm_detb_ne = (~(RTERM_DETB_1 | RTERM_DETB_2)) & rterm_detb_d;

//assign rterm_detb_lfps1 = lfps_det_3cycle_1 | (rterm_detb_enter_u0_en & rterm_detb_ne); 
//assign rterm_detb_lfps2 = lfps_det_3cycle_2 | (rterm_detb_enter_u0_en & rterm_detb_ne);

assign rterm_detb_lfps1 = lfps_det_3cycle_1 | (rterm_detb_ne); 
assign rterm_detb_lfps2 = lfps_det_3cycle_2 | (rterm_detb_ne);


/*
always @(posedge clk or negedge rstn)
begin
    if(!rstn)
    begin
        data_env_out_sync <= 2'b00;
    end
    else
    begin
        data_env_out_sync[0] <= DATA_ENV_OUT_1 | DATA_ENV_OUT_2;
        data_env_out_sync[1] <= data_env_out_sync[0];
    end
end
*/



usb_lanectl_1 u_usb_lanctl_1(
        .clk(clk),
        .clk_div4(clk),
        .rst_sel(1'b1),
	.txpulse_cnt_rstn(txpulse_cnt_rstn),
        .u2_en_r(RX_DET_CTL_2),
        .add_f_result(add_a_f_result),
        .sel_350_450(sel_350_450),
        .in_cnt_353ms_pulse (l2_353ms_pulse ),
	.out_nolfps_4ms_pulse (l1_nolfps_4ms_pulse),
        .out_cnt_353ms_pulse(l1_353ms_pulse ),
        .sel_4ms(1'b1),
        .clk_div6(clk_div6),
        .rstn(rstn),
         
        .u2_rxdet_en_sel(1'b0),
        .u2_rxdet_en_i(u2_rxdet_en),
        .u2_rxdet_en(u2_rxdet_en_1),
        .powerdown(powerdown),
        .data_env_en(data_env_en),
        .d240ms_disable(d240ms_disable),
        .nolfps_4ms_en (nolfps_4ms_en ),
        .cm_ctlb_ctl(cm_ctlb_ctl),
        .SLP_S0(SLP_S0),
        .nolfps_4ms_r_n   (l1_4ms_s   ),  
        .o_nolfps_4ms_r_n (l2_4ms_s   ),
        
        .rx_det_ctl_sel(rx_det_ctl_sel),

        .i2c_rxdet_period(i2c_rxdet_period),
        .i2c_u2_rxdet_interval(i2c_u2_rxdet_interval),
        .i2c_u2_period(i2c_u2_period),
        .u2_disable_rxdet(u2_disable_rxdet),
        .lfps_debounce_en(lfps_debounce_en),

        .Polling(Polling_1),
        .U2_EN(U2_EN_1),
        .DATA_ENV_OUT(usb_data_env1_syn),//(DATA_ENV_OUT_1),
        .data_env_sel(usb_data_env1_syn),
        .HS_EN(HS_EN_1),
        .HS_EN_O(HS_EN_2),
        .LS_EN(LS_EN_1),
        .TX_CMP(TX_CMP_1),

        .usb_en(usb_en),
        .lfps_anotherchannel(rterm_detb_lfps2),//(lfps_det_3cycle_2),
        .txtermdet_other(txcmp_2cycle_2),
        
        .tout_cnt_other(tout_cnt_2),

         .u2_nocmpdet_other(u2_nocmpdet_2),

        .lane1(lane1),
        .l1_lfps_det(l1_lfps_det),
        .lfps_det_dly(lfps_det_dly ),
        //.lfps_det_2(lfps_det_3cycle_1),
        .txcmp_2cycle(txcmp_2cycle_1),
        .tout_cnt(tout_cnt_1),
        .u2_nocmpdet(u2_nocmpdet_1),

        .LP_EN(LP_EN_1),

        .CM_CTLB(CM_CTLB_1),
        .vddl_ctl(vddl_ctl_1),
        .rterm_detb_ne0(rterm_detb_ne0),
        .RX_DET_CTL(RX_DET_CTL_1),
        .usb_rterm_cmp_off(usb_rterm_cmp_off),
        .RTERM_CMP_OFF(RTERM_CMP_OFF_1),
        .RX_GAIN_OFF(RX_GAIN_OFF_1),
        .RTERM_DETB(RTERM_DETB_1),
        .RTERM_DETB_BUF(RTERM_DETB_BUF_1),
        .HS_EN_BUF(HS_EN_BUF_1),
        .txpulse_cnt(txpulse_cnt),
        .det_ctl0(det_ctl0 ), 
        .det_ctl1(det_ctl1 ), 
        .det_ctl2(det_ctl2 ), 
        .det_ctl3(det_ctl3 ), 
        .det_ctl4(det_ctl4 ), 
        .det_ctl5(det_ctl5 ), 
        .det_ctl6(det_ctl6 )


);

usb_lanectl_0 u_usb_lanctl_2(
        .clk(clk),
        .clk_div4(clk_div16),
        .rst_sel(1'b0),
	.lfps_det_3cycle_1(lfps_det_3cycle_1),
	.txpulse_cnt_rstn(txpulse_cnt_rstn),
	.rterm_detb_n(rterm_detb_n),
        .add_f_result(add_b_f_result),
        .sel_350_450(sel_350_450),
        .pulse_350450_syn(pulse_350450_syn),
        .in_cnt_353ms_pulse (l1_353ms_pulse ),
	.in_nolfps_4ms_pulse (l1_nolfps_4ms_pulse),
        .out_cnt_353ms_pulse(l2_353ms_pulse ),
        .sel_4ms(1'b0),
        .clk_div6(clk_div6),
        .rstn(rstn),
         
        .u2_rxdet_en_sel(1'b1),
        .u2_rxdet_en_i(u2_rxdet_en),
        .u2_rxdet_en(u2_rxdet_en_2),
        .powerdown(powerdown),
        .data_env_en(data_env_en),
        .d240ms_disable(d240ms_disable),
        .nolfps_4ms_en (nolfps_4ms_en ),
        .SLP_S0(SLP_S0),
        .nolfps_4ms_r_n   (l2_4ms_s   ),  
        .o_nolfps_4ms_r_n (l1_4ms_s   ),
        .cm_ctlb_ctl(cm_ctlb_ctl),
        .rx_det_ctl_sel(rx_det_ctl_sel),

        .i2c_rxdet_period(i2c_rxdet_period),
        .i2c_u2_rxdet_interval(i2c_u2_rxdet_interval),
        .i2c_u2_period(i2c_u2_period),
        .u2_disable_rxdet(u2_disable_rxdet),
        .lfps_debounce_en(lfps_debounce_en),

        .Polling(Polling_2),
        .U2_EN(U2_EN_2),
        .usb_data_env_n   (usb_data_env_n),
        .usb_data_env1_syn(usb_data_env1_syn),
        .usb_data_env2_syn(usb_data_env2_syn),
        .DATA_ENV_OUT(usb_data_env2_syn),//(DATA_ENV_OUT_2),
        .data_env_sel(usb_data_env2_syn),//(1'b0),
        .HS_EN(HS_EN_2),
        .HS_EN_O(HS_EN_1),
        .LS_EN(LS_EN_2),
        .TX_CMP(tx_cmp_s),//(TX_CMP_2),

        .usb_en(usb_en),
        .lfps_anotherchannel(rterm_detb_lfps1),//(lfps_det_3cycle_1),
        .txtermdet_other(txcmp_2cycle_1),
        .tout_cnt_other(tout_cnt_1),
        .u2_nocmpdet_other(u2_nocmpdet_1),

        .lane1(lane2),
        .lfps_det_2(lfps_det_3cycle_2),
        
        .txcmp_2cycle(txcmp_2cycle_2),
        .tout_cnt(tout_cnt_2),
        .u2_nocmpdet(u2_nocmpdet_2),


        .LP_EN(LP_EN_2),

        .CM_CTLB(CM_CTLB_2),
        .vddl_ctl(vddl_ctl_2),
        .rterm_detb_ne0(rterm_detb_ne0),
        .RX_DET_CTL(RX_DET_CTL_2),
        .usb_rterm_cmp_off(usb_rterm_cmp_off),
        .RTERM_CMP_OFF(RTERM_CMP_OFF_2),
        .RX_GAIN_OFF(RX_GAIN_OFF_2),
        .RTERM_DETB(RTERM_DETB_2),//(rterm_detb_s),   //
        .RTERM_DETB_BUF(RTERM_DETB_BUF_2),
        .HS_EN_BUF(HS_EN_BUF_2)

);

cm_active u_cm_active (
        .clk_div6(clk),
        .rstn(rstn),

        .powerdown(powerdown),

        .lfps1(lfps_det_3cycle_1),
        .lfps2(lfps_det_3cycle_2),
        .clear(RX_GAIN_OFF), //RTERM_DETB_1 & RTERM_DETB_2),
        .cm_active(cm_active)
        );

endmodule

